local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local borders = import('/lua/ui/game/borders.lua').GetMapGroup()
local ToolTip = import('/lua/ui/game/tooltip.lua')
local EffectHelpers = import('/lua/maui/effecthelpers.lua')
local GPGSelection = import('/lua/ui/game/selection.lua')
local selectionSets = GPGSelection.selectionSets

CurrBattalion = 0
CurrSquad = false
local panel = false
local mouseover = false

function SetCurrBattalion(num)
	CurrBattalion = num
    CurrSquad = false
	RefreshDisplay()
end

function SetCurrSquad(num)
	CurrSquad = num
	RefreshDisplay()
end

function RefreshDisplay()
	local batt = "X"
	local squad = "X"
	if CurrBattalion then
		batt = CurrBattalion
	end
	if CurrSquad then
		squad = CurrSquad
	end
	
	CurrBattalionDisplay:SetText(batt .. " : " .. squad)
end

function MakeSquad(num)
	-- Create squad [num] in current battalion
	-- Format for label is currBatt:squadNum
	clearSquad(GetSelectedUnits())
	ConExecute("UI_MakeSelectionSet " .. GetSquadLabel(num))
	SetCurrSquad(num)
end

function AppendToSquad(num)
   -- Insert selected units into squad
   local totalSel = GetSelectedUnits()
   label = GetSquadLabel(num)
   selectionSets[label] = ValidateUnitsList(selectionSets[label])
   local selectionSet = selectionSets[label]
   if selectionSet then
      for i, unit in selectionSet do
         table.insert(totalSel, unit)
      end
   end    
   SelectUnits(totalSel)
   clearSquad(totalSel)

   ConExecute("UI_MakeSelectionSet " .. GetSquadLabel(num))
   SetCurrSquad(num)
end

function FilterToBattalion()
   -- Reduce selection to only those units in battalion
   local totalSel = GetSelectedUnits()
   totalSel = ValidateUnitsList(totalSel)
   local finalSel = {}
   local label
   local selectionSet = EntityCategoryFilterDown(categories.ALLUNITS - categories.FACTORY, totalSel)
   if selectionSet then
      for i, unit in selectionSet do         
         for i=1,4 do
            label = GetSquadLabel(i)
            if unit:HasSelectionSet(label) then
               table.insert(finalSel, unit)
            end 
         end
      end
   end    
   SelectUnits(finalSel)
   SetCurrSquad(false)
end


function SelectSquad(num)
    local label = GetSquadLabel(num)
    local selUnits = GetSelectedUnits()
    if ((num == CurrSquad) and selUnits) then
       -- We're trying to reselect the current squad
       if table.getn(selUnits) > 1 then
          -- Got a group selected. Go to first unit
          selectFirstUnit(label)
       elseif table.getn(selUnits) == 1 then
          selectNextUnit(label, selUnits[1])             
       else
          -- WTF? Shouldnt happen...
          LOG("SelectSquad ERROR: selUnits = " .. table.getn(selUnits))
       end    
    else
       -- Selecting squad for the first time
       GPGSelection.FactorySelection("") -- Shitty way to clear their doubletap code
       ConExecute("UI_ApplySelectionSet " .. label)
       if GetSelectedUnits() then 
          SetCurrSquad(num)
       else
          SetCurrSquad(false)
       end
    end
end

function SelectAllSquads(battNum)
   local totalSel = {}
   for i=1,4 do
      label = getLabel(battNum, i)
      selectionSets[label] = ValidateUnitsList(selectionSets[label])
      local selectionSet = EntityCategoryFilterDown(categories.ALLUNITS - categories.FACTORY, selectionSets[label])
      if selectionSet then
         for i, unit in selectionSet do
            table.insert(totalSel, unit)
         end
      end    
   end
   SelectUnits(totalSel)
end

function SelectBattalion(num)
--    Commented: dont bother with squad switching, its not useful.
--    if ((CurrBattalion ~= num) or (CurrSquad == false)) then
      SetCurrBattalion(num)
      SelectAllSquads(num)
      if GetSelectedUnits() then
         SetCurrSquad("X")
      end   
--    elseif CurrSquad == "1-4" then
--       SelectSquad(1)      
--    elseif CurrSquad < 5 then
--       SelectSquad(CurrSquad + 1)
--    else
--       SelectUnits({})
--       SetCurrSquad(false)
--    end  
end

function FillBattalion(num)
   SetCurrBattalion(num)

   local sel = GetSelectedUnits()
   local sortTables = {{},{},{},{}}

   if sel then
      -- If we have units, then sort them into types
      table.sort(sel, 
                 function(unit1, unit2)
                    bpid1 = unit1:GetBlueprint().BlueprintId
                    bpid2 = unit2:GetBlueprint().BlueprintId
                    return bpid1<bpid2
                 end)

      -- Split them into squads
      local index = 1
      for i, unit in sel do         
         table.insert(sortTables[index], unit)
         if index < 4 then
            index = index + 1
         else
            index = 1
         end
      end
      
      -- Add them to squads
      for i=1, 4 do
         SelectUnits(sortTables[i])
         MakeSquad(i)
      end
      SelectBattalion(num)
   else
      -- If no units selected, clear the battalion
      SelectBattalion(num)
      clearSquad(GetSelectedUnits())
   end  
end

function FillBattalionByType(num)
   SetCurrBattalion(num)

   local sel = GetSelectedUnits()
   local sortTables = {{},{},{},{}}

   if sel then
      -- If we have units, then sort them into types
      table.sort(sel, 
                 function(unit1, unit2)
                    bpid1 = unit1:GetBlueprint().BlueprintId
                    bpid2 = unit2:GetBlueprint().BlueprintId
                    return bpid1<bpid2
                 end)

      -- Split them into squads
      local index = 1
      bpid1 = sel[1]:GetBlueprint().BlueprintId

      for i, unit in sel do         
         bpid2 = unit:GetBlueprint().BlueprintId
         if bpid1 ~= bpid2 then
            bpid1 = bpid2
            if index < 4 then
               index = index + 1
            else
               index = 1
            end
         end    

         table.insert(sortTables[index], unit)
      end
      
      -- Add them to squads
      for i=1, 4 do
         SelectUnits(sortTables[i])
         MakeSquad(i)
      end
      SelectBattalion(num)
   else
      -- If no units selected, clear the battalion
      SelectBattalion(num)
      clearSquad(GetSelectedUnits())
   end  
end


function GetSquadLabel(num)
	return getLabel(CurrBattalion, num)
end

function GetCurrentLabel()
	return getLabel(CurrBattalion, CurrSquad)
end

function AppendSetToSelection(num)
   GPGSelection.AppendSetToSelection(GetSquadLabel(num))
end

function FactorySelection(num)
   GPGSelection.FactorySelection(GetSquadLabel(num))
end

function clearSquad(unitList)
	if unitList then
		for i, unit in unitList do
			local setList = unit:GetSelectionSets()	
			for i, set in setList do			
				unit:RemoveSelectionSet(set)
                removeUnitFromSelectionSet(unit, set)
			end
		end
	end
end

function removeUnitFromSelectionSet(unit, set)
   if unit and set then
      for i, currUnit in selectionSets[set] do
        if currUnit == unit then
           table.remove(selectionSets[set], i)
        end 
      end
   end
end

function getLabel(battalion, squad)
	return battalion .. ":" .. squad
end

function selectFirstUnit(name)
   if not selectionSets[name] then return end
   selectionSets[name] = ValidateUnitsList(selectionSets[name])
   local selection = EntityCategoryFilterDown(categories.ALLUNITS - (categories.FACTORY - categories.MOBILE) , selectionSets[name])
   if not selection then return end

   local tempTable = {}
   table.insert(tempTable, selection[1])
   SelectUnits(tempTable)
end

function selectNextUnit(name, unit)
   if not selectionSets[name] then return end
   selectionSets[name] = ValidateUnitsList(selectionSets[name])
   local selection = EntityCategoryFilterDown(categories.ALLUNITS - (categories.FACTORY - categories.MOBILE) , selectionSets[name])
   if not selection then return end

   -- Look for the unit and select the next one
   local found = false
   local tempTable = {}
   for i, currUnit in selection do
      if currUnit == unit then
         table.insert(tempTable, selection[i+1])
         SelectUnits(tempTable)
         found = true
      end 
   end

   if not found then
      SelectUnits(tempTable)
   end  
end

-------------------------------------------
function CreatePanel(parent)
	local panel = Bitmap(parent)
	panel:SetTexture('/mods/tieredgrouping/textures/panel.dds')
	panel.Right:Set(function() return parent.Right() - 19 end)
	panel.Top:Set(function() return parent.Top() + 5 end)
	panel.Depth:Set(100)
	panel.HandleEvent = function(self, event)
		if event.Type == 'MouseExit' then
			CloseToolTip()
		end
	end
	return panel
end

function CreateSplitControls(parent)
	local splitgroup = Group(parent)
	splitgroup.Left:Set(parent.Left)
	splitgroup.Top:Set(function() return parent.Top() + 5 end)
	splitgroup.Right:Set(parent.Right)
	splitgroup.Height:Set(30)
	local title = UIUtil.CreateText(splitgroup, "X : X", 30)
	CurrBattalionDisplay = title
	title.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Battalion : Squad", Body = "Tiered Grouping by AdmiralZeech: This panel displays the current Battalion and Squad"})
		end
	end
	LayoutHelpers.AtLeftTopIn(title, splitgroup, 5, 1)
	return splitgroup
end

function CreateToolTip(control, data, togglebutton)
	if mouseover then mouseover:Destroy() mouseover = false end
	local text = data.Title
	local body = data.Body
	mouseover = ToolTip.CreateExtendedToolTip(control, text, body)
	mouseover.Top:Set(function() return panel.Top() + 2 end)
	mouseover.Right:Set(function() return panel.Left() - 1 end)
	mouseover:SetAlpha(0, true)
	mouseover:SetNeedsFrameUpdate(true)
	
	local delayTime = 0.5
	local totaltime = 0
	mouseover.OnFrame = function(self, deltaTime)
		if totaltime > delayTime then
			local alpha = self:GetAlpha() + (deltaTime * 4)
			if alpha < 1 then
				self:SetAlpha(alpha, true)
			else
				self:SetAlpha(1, true)
				self:SetNeedsFrameUpdate(false)
			end
		end
		totaltime = totaltime + deltaTime
	end
end
		
function CloseToolTip()
	if mouseover then mouseover:Destroy() mouseover = false end
end

function hideEffect(control, subcontrol1)
	control:DisableHitTest(true)
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.1
	local time2 = 0.2
	local minHeight = 16
    local timeAccum = 0

	EffectHelpers.FadeOut(subcontrol1, timefade)
	
    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
            self:SetNeedsFrameUpdate(false)
            self:Hide()
			self.Left:Set(function() return self.Right() - 3 end)
		elseif timeAccum >= time1 then
			self.Left:Set(function() return self.Right() - self.BitmapWidth()*((time2+time1 - timeAccum)/(time2)) end)
			self.Bottom:Set(function() return self.Top() + minHeight end)
		else
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((time1 - timeAccum)/(time1)) end )
		end
    end
end

function showEffect(control, subcontrol1)
	control:Show()
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.2
	local time2 = 0.1
	local minHeight = 16
    local timeAccum = 0

    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
			self:EnableHitTest(true)
            self:SetNeedsFrameUpdate(false)
			LayoutHelpers.ResetLeft(self)
			LayoutHelpers.ResetBottom(self)
			EffectHelpers.FadeIn(subcontrol1, timefade)
		elseif timeAccum >= time1 then
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((timeAccum - time1)/(time2)) end )
			self.Left:Set(function() return self.Right() - self.BitmapWidth() end)
        else
			self.Left:Set(function() return self.Right() - self.BitmapWidth()*((timeAccum)/(time1)) end)
        end
    end
end

function Init()
	panel = CreatePanel(borders)
	splitPanel = CreateSplitControls(panel)
	panelToggle = UIUtil.CreateCheckbox(borders, 
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_dis.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_dis.dds'))
    LayoutHelpers.AtRightTopIn(panelToggle, borders, 3, 5)
	panelToggle.Height:Set(16)
	panelToggle.Depth:Set(100)
    panelToggle.OnCheck = function(self, checked)
        if checked then
			hideEffect(panel, splitPanel)
		else
			showEffect(panel, splitPanel)
		end
    end
	panelToggle.oldHandleEvent = panelToggle.HandleEvent
	panelToggle.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			CloseToolTip()
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the selection panel."}, true)
			else
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the selection panel."}, true)
			end
		elseif event.Type == 'MouseEnter' then
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the selection panel."}, true)
			else
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the selection panel."}, true)
			end
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
		panelToggle.oldHandleEvent(self, event)
	end

	RefreshDisplay()
	local newSelectionsMap = {
    ['Ctrl-Alt-1']      = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FactorySelection(1)'}, 
	['Ctrl-Alt-2']		= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FactorySelection(2)'}, 
	['Ctrl-Alt-3']		= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FactorySelection(3)'}, 
	['Ctrl-Alt-4']		= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FactorySelection(4)'}, 
	['Ctrl-Alt-5']		= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FactorySelection(5)'}, 
	['G']               = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FilterToBattalion()'},
	}
	IN_AddKeyMapTable(newSelectionsMap)

	local defaultKeyMap = import("/lua/keymap/defaultKeyMap.lua").defaultKeyMap
	defaultKeyMap['1']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectSquad(1)'}
	defaultKeyMap['2']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectSquad(2)'}
	defaultKeyMap['3']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectSquad(3)'}
	defaultKeyMap['4']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectSquad(4)'}
	defaultKeyMap['5']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectSquad(5)'}
	defaultKeyMap['6']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectBattalion(6)'}
	defaultKeyMap['7']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectBattalion(7)'}
	defaultKeyMap['8']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectBattalion(8)'}
	defaultKeyMap['9']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectBattalion(9)'}
	defaultKeyMap['0']              = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SelectBattalion(0)'}       
	defaultKeyMap['Ctrl-1']         = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").MakeSquad(1)'}
	defaultKeyMap['Ctrl-2']         = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").MakeSquad(2)'}
	defaultKeyMap['Ctrl-3']         = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").MakeSquad(3)'}
	defaultKeyMap['Ctrl-4']         = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").MakeSquad(4)'}
	defaultKeyMap['Ctrl-5']         = {action =  'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").MakeSquad(5)'}
	defaultKeyMap['Ctrl-6']	        = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalionByType(6)'}
	defaultKeyMap['Ctrl-7']	        = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalionByType(7)'}
	defaultKeyMap['Ctrl-8']	        = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalionByType(8)'}
	defaultKeyMap['Ctrl-9']	        = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalionByType(9)'}
	defaultKeyMap['Ctrl-0']	        = {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalionByType(0)'}
	defaultKeyMap['Shift-1']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendToSquad(1)'}
	defaultKeyMap['Shift-2']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendToSquad(2)'}
	defaultKeyMap['Shift-3']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendToSquad(3)'}
	defaultKeyMap['Shift-4']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendToSquad(4)'}
	defaultKeyMap['Shift-5']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendToSquad(5)'}
	defaultKeyMap['Shift-6']		= {action =	'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SetCurrBattalion(6)'}
	defaultKeyMap['Shift-7']		= {action =	'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SetCurrBattalion(7)'}
	defaultKeyMap['Shift-8']		= {action =	'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SetCurrBattalion(8)'}
	defaultKeyMap['Shift-9']		= {action =	'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SetCurrBattalion(9)'}
	defaultKeyMap['Shift-0']		= {action =	'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").SetCurrBattalion(0)'}
	defaultKeyMap['Ctrl-Shift-1']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendSetToSelection(1)'}
	defaultKeyMap['Ctrl-Shift-2']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendSetToSelection(2)'}
	defaultKeyMap['Ctrl-Shift-3']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendSetToSelection(3)'}
	defaultKeyMap['Ctrl-Shift-4']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendSetToSelection(4)'}
	defaultKeyMap['Ctrl-Shift-5']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").AppendSetToSelection(5)'}
	defaultKeyMap['Ctrl-Shift-6']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalion(6)'}
	defaultKeyMap['Ctrl-Shift-7']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalion(7)'}
	defaultKeyMap['Ctrl-Shift-8']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalion(8)'}
	defaultKeyMap['Ctrl-Shift-9']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalion(9)'}
	defaultKeyMap['Ctrl-Shift-0']	= {action =	 'UI_Lua import("/mods/tieredgrouping/groupinterface.lua").FillBattalion(0)'}

end
